@@ -1,7 +1,7 @@
 package Catalyst::View;
 
-use strict;
-use base qw/Catalyst::Component/;
+use Moose;
+extends qw/Catalyst::Component/;
 
 =head1 NAME
 
@@ -46,16 +46,9 @@ sub process {
 
 }
 
-=head2 $c->merge_hash_config( $hashref, $hashref )
+=head1 AUTHORS
 
-Merges two hashes together recursively, giving right-hand precedence.
-
-=cut
-
-=head1 AUTHOR
-
-Sebastian Riedel, C<sri@oook.de>
-Marcus Ramberg, C<mramberg@cpan.org>
+Catalyst Contributors, see Catalyst.pm
 
 =head1 COPYRIGHT
 
@@ -64,4 +57,7 @@ the same terms as Perl itself.
 
 =cut
 
+no Moose;
+__PACKAGE__->meta->make_immutable();
+
 1;
